/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package vezerles;

import alaposztalyok.Jatekos;
import alaposztalyok.Pillango;
import alaposztalyok.Vadasz;
import felulet.JatekPanel;
import felulet.JatekosPanel;
import felulet.KijelzoPanel;
import java.awt.Graphics;
import java.awt.Image;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.ImageIcon;

/**
 *
 * @author :)
 */
// Lehetne Thread is, de akkor nem tudnánk többször újra-indítani.
// Ebből viszont lehet többször is létrehozni egy-egy szálat.
public class Vezerlo implements Runnable{       
        
    private final long JATEK_IDO_SEC = 5;
    private final int ALSO_IDO_MSEC = 10;
    private final int FELSO_IDO_MSEC = 50;
    private final int ALSO_PILLANGO_MERET_PIXEL = 50;
    private final int FELSO_PILLANGO_MERET_PIXEL = 100;
    private final int KEP_SZAM = 10;
    private final long KELETKEZESI_IDO_MSEC = 1000;
    private final int VADASZ_SZELESSEG = 150;
    private final int VADASZ_MAGASSAG = 150;
    
    private JatekPanel jatekPanel;
    private KijelzoPanel kijelzoPanel;
    private JatekosPanel jatekosPanel;
    
    private List<Image> kepek = new ArrayList<>();
    private List<Pillango> pillangok = new CopyOnWriteArrayList<>();
    private Vadasz vadasz;
    private Jatekos jatekos;

    public Vezerlo(JatekPanel jatekPanel, KijelzoPanel kijelzoPanel, JatekosPanel jatekosPanel) {
        this.jatekPanel = jatekPanel;
        this.kijelzoPanel = kijelzoPanel;
        this.jatekosPanel = jatekosPanel;
    }
    
    public void beallitas() {
        
        // Beállítja a vadász méretét.
        Vadasz.setKepSzelesseg(VADASZ_SZELESSEG);
        Vadasz.setKepMagassag(VADASZ_MAGASSAG);  
        
        // Beállítja a felület méretét.
        Pillango.setFeluletSzelesseg(jatekPanel.getWidth());
        Pillango.setFeluletMagassag(jatekPanel.getWidth());
        
        kepFeltoltes();
    }    
    
    public void kepFeltoltes(){

        for (int i = 1; i <= KEP_SZAM; i++) {
            kepek.add(new ImageIcon(this.getClass().
                    getResource("/kepek/pillango"+i+".gif")).getImage());
        }       
    }

    @Override
    public void run() {
        long hatralevoIdo = JATEK_IDO_SEC * 1000;
        while (hatralevoIdo >= 0) {
            try {
                kijelzoPanel.idotKiir(hatralevoIdo / 1000);
                pillangotIndit();
                Thread.sleep(KELETKEZESI_IDO_MSEC);
                hatralevoIdo -= KELETKEZESI_IDO_MSEC;
            } catch (InterruptedException ex) {
                Logger.getLogger(Vezerlo.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        kijelzoPanel.jatekVeg();
        jatekosPanel.jatekVeg();
        leallit();
    }    

    private void pillangotIndit() {
        int kepSzelesseg = (int) (Math.random()
                        *(FELSO_PILLANGO_MERET_PIXEL-ALSO_PILLANGO_MERET_PIXEL)
                        + ALSO_PILLANGO_MERET_PIXEL);
        int kepMagassag = (int) (Math.random()
                        *(FELSO_PILLANGO_MERET_PIXEL-ALSO_PILLANGO_MERET_PIXEL)
                        + ALSO_PILLANGO_MERET_PIXEL);       
        int kepX = (int) (Math.random()*(jatekPanel.getWidth() - kepSzelesseg));
        int kepY = (int) (Math.random()*(jatekPanel.getHeight() - kepMagassag));
        long ido = (long) (Math.random()*(FELSO_IDO_MSEC - ALSO_IDO_MSEC) 
                        + ALSO_IDO_MSEC);
        int kepIndex = (int) (Math.random()*kepek.size());
        Image kep = kepek.get(kepIndex);
        int dx = (Math.random() < 0.5) ? 1 : -1;
        int dy = (Math.random() < 0.5) ? 1 : -1;
        
        Pillango pillango = new Pillango(kep, kepX, kepY, 
                                         kepSzelesseg, kepMagassag, 
                                         ido, dx, dy, this);
        pillangok.add(pillango);
        pillango.start();
    }    
    
    public void frissit() {
        jatekPanel.repaint();
    }    

    public void rajzolas(Graphics g) {
        for (Pillango pillango : pillangok) {
            pillango.rajzolas(g);
        }
        
        if(vadasz != null){
            vadasz.rajzolas(g);
        }
    }

    public void jatekInditas(Jatekos jatekos) {
        
        this.jatekos = jatekos;
        
        vadasz = new Vadasz(jatekPanel.getWidth()-Vadasz.getKepSzelesseg()/2,
                            jatekPanel.getHeight()-Vadasz.getKepMagassag()/2);
        frissit();
        
        // Létrehoz és elindít egy új szálat,
        // ez a szál hozza létre a pillangókat
        Thread szal = new Thread(this);
        szal.start();
        kijelzoPanel.jatekKezdet();
    }

    public void torol(Pillango pillango) {
        pillangok.remove(pillango);
        jatekos.pontotKap();
        kijelzoPanel.pontszamotKiir(jatekos.getPontSzam());
    }

    private void leallit() {
        // Ez a megoldás engedi tovább futni a szálakat, amíg maguktól meg nem állnak
        pillangok.clear();
        vadasz = null;
        frissit();

    }

    public void vadasztArrebbMozgat(int x, int y) {
        if(vadasz != null && vadasz.megfogtak(x,y)){
            
            vadasz.setKepX(x);
            vadasz.setKepY(y);
            
            int haloKozepX = x - Vadasz.getKepSzelesseg() / 4;
            int haloKozepY = y - Vadasz.getKepMagassag() / 4;
            
            lepkeFogas(haloKozepX, haloKozepY);
            
        }
    }

    private void lepkeFogas(int x, int y) {
        for (Pillango pillango : pillangok) {
            pillango.megfogtak(x,y);
        }
    }

}

